@@ -9,14 +9,16 @@ from io import StringIO
 from pathlib import Path
 from traceback import format_exception
 from warnings import warn
+import signal
 
-import cv2 as cv
 import docstring_parser as dp
 import pandas as pd
+import pyqtgraph as pg
 from pyqtgraph.Qt import QtWidgets, QtCore
 from pyqtgraph.parametertree import Parameter, ParameterTree
 from ruamel.yaml import YAML, YAMLError
 from tqdm import tqdm
+import cv2 as cv
 
 from .params.prjparam import PrjParam
 from .typeoverloads import FilePath
@@ -151,6 +153,7 @@ def makeExceptionsShowDialogs(win: QtWidgets.QMainWindow,
     dlg.show()
     dlg.exec_()
   def patch_excepthook():
+    signal.signal(signal.SIGINT, signal.SIG_DFL)
     global usingPostponedErrors
     sys.excepthook = new_except_hook
     usingPostponedErrors = True
@@ -192,10 +195,7 @@ def pascalCaseToTitle(name: str, addSpaces=True) -> str:
     return name
   if name.startswith('FR'):
     name = name[2:]
-  if addSpaces:
-    replace = r'\1 \2'
-  else:
-    replace = r'\1\2'
+  replace = r'\1 \2' if addSpaces else r'\1\2'
   name = re.sub(r'(\w)([A-Z])', replace, name)
   name = name.replace('_', ' ')
   return name.title().strip()
@@ -227,7 +227,8 @@ def clsNameOrGroup(cls: t.Union[type, t.Any]):
   return nameFormatter(cls.__name__)
 
 
-def saveToFile(saveObj, savePath: Path, allowOverwriteDefault=False):
+def saveToFile(saveObj, savePath: FilePath, allowOverwriteDefault=False):
+  savePath = Path(savePath)
   if not allowOverwriteDefault and savePath.stem.lower() == 'default':
     errMsg = 'Cannot overwrite default setting.\n\'Default\' is automatically' \
              ' generated, so it should not be modified.'
@@ -317,9 +318,7 @@ def applyParamOpts(param: Parameter, opts: dict):
   state = opts.copy()
   childStates = state.pop('children', [])
   if isinstance(childStates, list):
-    cs = {}
-    for child in childStates:
-      cs[child['name']] = child
+    cs = {child['name']: child for child in childStates}
     childStates = cs
   param.setOpts(**opts)
   for chName, chDict in childStates.items():
@@ -335,18 +334,33 @@ def params_flattened(param: Parameter):
   return addList
 
 
-def flexibleParamTree(topParam:Parameter, showTop=True):
+def flexibleParamTree(treeParams: t.Union[t.List[Parameter], Parameter]=None, showTop=True, setTooltips=True):
   tree = ParameterTree()
   tree.setTextElideMode(QtCore.Qt.ElideRight)
   tree.header().setSectionResizeMode(QtWidgets.QHeaderView.Interactive)
-  tree.setParameters(topParam, showTop)
-  def hookupSignals(p: Parameter):
-    for ch in p:
-      hookupSignals(ch)
-    p.sigChildAdded.connect(lambda _p, _ch: hookupSignals(_ch))
+  if isinstance(treeParams, Parameter):
+    treeParams = [treeParams]
+  if not treeParams:
+    treeParams = []
+  # pyqtgraph bug: If tree isn't explicitly cleared, the inivisble root isn't configured correctly
+  for param in treeParams:
+    tree.addParameters(param, root=None, showTop=showTop)
+  # def hookupSignals(p: Parameter):
+  #   for ch in p:
+  #     hookupSignals(ch)
+  # Make wrapper out here to avoid lambda in loop scoping issues
+  def hookupWrapper(param):
+    def maybeUpdateTips(_param, change):
+      desc = change[0][1]
+      if 'added' in desc.lower():
+        setParamTooltips(tree)
+    param.sigTreeStateChanged.connect(maybeUpdateTips)
 
-  setParamTooltips(tree)
-  hookupSignals(topParam)
+  if setTooltips:
+    for param in treeParams:
+      hookupWrapper(param)
+    setParamTooltips(tree)
+  # hookupSignals(topParam)
   return tree
 
 def setParamsExpanded(tree: ParameterTree, expandedVal=True):
@@ -475,6 +489,39 @@ def paramWindow(param: Parameter):
   layout.addWidget(tree)
   dlg.exec_()
 
+def hookupParamWidget(param: Parameter, widget):
+  """
+  Parameter widgets created outside param trees need to have their sigChanging, sigChanged, etc. signals hooked up to
+  the parameter itself manually. The relevant code below was extracted from WidgetParameterItem
+  """
+  def widgetValueChanged():
+    val = widget.value()
+    return param.setValue(val)
+  def paramValueChanged(param, val, force=False):
+    if force or not pg.eq(val, widget.value()):
+      try:
+        widget.sigChanged.disconnect(widgetValueChanged)
+        param.sigValueChanged.disconnect(paramValueChanged)
+        widget.setValue(val)
+        param.setValue(widget.value())
+      finally:
+        widget.sigChanged.connect(widgetValueChanged)
+        param.sigValueChanged.connect(paramValueChanged)
+
+  param.sigValueChanged.connect(paramValueChanged)
+  if widget.sigChanged is not None:
+    widget.sigChanged.connect(widgetValueChanged)
+
+  if hasattr(widget, 'sigChanging'):
+    widget.sigChanging.connect(lambda: param.sigValueChanging.emit(param, widget.value()))
+
+  ## update value shown in widget.
+  opts = param.opts
+  if opts.get('value', None) is not None:
+    paramValueChanged(param, opts['value'], force=True)
+  else:
+    ## no starting value was given; use whatever the widget has
+    widgetValueChanged()
 
 def popupFilePicker(parent=None, winTitle: str='', fileFilter: str='', existing=True, asFolder=False,
                     selectMultiple=False, startDir: str=None, **kwargs) -> t.Optional[t.Union[str, t.List[str]]]:
@@ -560,7 +607,7 @@ def hierarchicalUpdate(curDict: dict, other: dict, replaceLists=False):
   if other is None:
     return curDict
   for k, v in other.items():
-    curVal = curDict.get(k, None)
+    curVal = curDict.get(k)
     if isinstance(curVal, dict) and isinstance(v, dict):
       hierarchicalUpdate(curVal, v, replaceLists)
     elif not replaceLists and isinstance(curVal, list) and isinstance(v, list):
