@@ -3,20 +3,17 @@ import weakref
 from contextlib import contextmanager
 from enum import Flag, auto
 from pathlib import Path
-from warnings import warn
 
 from pyqtgraph.Qt import QtCore, QtWidgets
 from pyqtgraph.parametertree import Parameter
 from pyqtgraph.parametertree.parameterTypes import GroupParameter
-from utilitys.widgets import PopupLineEditor
 
+from utilitys.widgets import EasyWidget
 from . import PrjParam
 from .procwrapper import NestedProcWrapper
 from .. import fns
 from ..processing import *
 from ..typeoverloads import FilePath
-import utilitys
-from ..fns import warnLater
 
 __all__ = [
   'RunOpts', 'ParamEditor', 'ParamEditorDockGrouping', 'EditorPropsMixin', 'SPAWNED_EDITORS'
@@ -70,7 +67,7 @@ class ParamEditor(QtWidgets.QDockWidget):
   """
 
   def __init__(self, parent=None, paramList: t.List[t.Dict] = None, saveDir: FilePath=None,
-               fileType='param', name=None, topTreeChild: Parameter = None,
+               fileType='param', name=None, treeChildren: t.Union[Parameter, t.List[Parameter]] = None,
                **kwargs):
     """
     GUI controls for user-interactive parameters within a QtWidget (usually main window). Each window consists of
@@ -84,8 +81,11 @@ class ParamEditor(QtWidgets.QDockWidget):
     :param fileType: The filetype of the saved settings. E.g. if a settings configuration
       is saved with the name "test", it will result in a file "test.&lt;fileType&gt;"
     :param name: User-readable name of this parameter editor
-    :param topTreeChild: Generally for internal use. If provided, it will
-      be inserted into the parameter tree instead of a newly created parameter.
+    :param topTreeChild: List of tree parameters. Since the default pyqtgraph TreeWidget doesn't easily allow inserting
+      a new parameter *before* an existing one, this allows specifying a different ordering of top-level tree children.
+      Since self.params will need to be specified in almost all cases, use *None* to specify where in the list it
+      should occur. In most cases, this can be left as *None* in which case self.params will be the top and only tree
+      child on startup.
     """
     super().__init__(parent)
     self.hide()
@@ -140,10 +140,14 @@ class ParamEditor(QtWidgets.QDockWidget):
 
     self.params.sigStateChanged.connect(self._paramTreeChanged)
 
-    topParam = self.params
-    if topTreeChild is not None:
-      topParam = topTreeChild
-    self.tree = fns.flexibleParamTree(topParam, showTop=False)
+    if treeChildren is None:
+      treeChildren = self.params
+    if isinstance(treeChildren, Parameter):
+      treeChildren = [treeChildren]
+    for ii, ch in enumerate(treeChildren):
+      if not ch:
+        treeChildren[ii] = self.params
+    self.tree = fns.flexibleParamTree(treeChildren, showTop=False)
 
     self._stateBeforeEdit = self.params.saveState()
     self.lastAppliedName = None
@@ -169,13 +173,13 @@ class ParamEditor(QtWidgets.QDockWidget):
       self.sigChangesApplied.emit(outDict)
     return outDict
 
-  def saveParamValues(self, saveName: str=None, paramState: dict=None, allowOverwriteDefault=False,
-                      includeDefaults=False, blockWrite=False):
+  def saveParamValues(self, saveName: str=None, paramState: dict=None, *, allowOverwriteDefault=False,
+                      includeDefaults=False, blockWrite=False) -> t.Optional[t.Dict[str, t.Any]]:
     """
     * Returns dict on successful parameter save and emits sigParamStateCreated.
     * Returns None if no save name was given
     """
-    if saveName is None or self.saveDir is None:
+    if (saveName is None or self.saveDir is None) and not blockWrite:
       return None
     elif paramState is None:
       paramState = fns.paramValues(self.params, includeDefaults=includeDefaults or allowOverwriteDefault)
@@ -202,6 +206,35 @@ class ParamEditor(QtWidgets.QDockWidget):
     self.lastAppliedName = stateName
     return loadDict
 
+  def _parse_updateCandidateParams(self, loadDict: dict, candidateParams: t.Sequence[Parameter]):
+    def validName(param, name):
+      return name in (param.opts['title'], param.name())
+    def checkParentChain(param, name):
+      if not param:
+        return False
+      return validName(param, name) or checkParentChain(param.parent(), name)
+
+    unhandled = {}
+    # Copy for mutable object
+    for kk, vv in loadDict.items():
+      if isinstance(vv, dict):
+        # Successively traverse down child tree
+        curCandidates = [p for p in candidateParams if checkParentChain(p, kk)]
+        # By this point defaults are already populated so no need to reset them otherwise infinite recursion will occur
+        self.loadParamValues('', vv, useDefaults=False, candidateParams=curCandidates, applyChanges=False)
+      else:
+        unhandled[kk] = vv
+    with self.params.treeChangeBlocker():
+      for kk, vv in unhandled.items():
+        matches = [p for p in candidateParams if validName(p, kk)]
+        if len(matches) == 1:
+          matches[0].setValue(vv)
+        elif len(matches) == 0:
+          fns.warnLater(f'No matching parameters for key {kk}. Ignoring.', UserWarning)
+        else:
+          raise ValueError(f'Multiple matching parameters for key {kk}:\n'
+                           f'{matches}')
+
   def loadParamValues(self, stateName: t.Union[str, Path],
                       stateDict: dict=None,
                       useDefaults=True,
@@ -227,43 +260,21 @@ class ParamEditor(QtWidgets.QDockWidget):
 
     defaultFile = self.formatFileName('Default')
     if useDefaults and defaultFile and defaultFile.exists():
-      base = fns.attemptFileLoad(defaultFile)
+      base = fns.attemptFileLoad(defaultFile) or {}
       loadDict = fns.hierarchicalUpdate(base, loadDict, replaceLists=True)
 
     if candidateParams is None:
       candidateParams = fns.params_flattened(self.params)
-      
-    def validName(param, name):
-      return name in (param.opts['title'], param.name())
-    def checkParentChain(param, name):
-      if not param:
-        return False
-      return validName(param, name) or checkParentChain(param.parent(), name)
-    
-    unhandled = {}
-    # Copy for mutable object
-    for kk, vv in loadDict.items():
-      if isinstance(vv, dict):
-        # Successively traverse down child tree
-        curCandidates = [p for p in candidateParams if checkParentChain(p, kk)]
-        # By this point defaults are already populated so no need to reset them otherwise infinite recursion will occur
-        self.loadParamValues('', vv, useDefaults=False, candidateParams=curCandidates, applyChanges=False)
-      else:
-        unhandled[kk] = vv
-    with self.params.treeChangeBlocker():
-      for kk, vv in unhandled.items():
-        matches = [p for p in candidateParams if validName(p, kk)]
-        if len(matches) == 1:
-          matches[0].setValue(vv)
-        elif len(matches) == 0:
-          warnLater(f'No matching parameters for key {kk}. Ignoring.', UserWarning)
-        else:
-          raise ValueError(f'Multiple matching parameters for key {kk}:\n'
-                           f'{matches}')
+
+    if candidateParams:
+      self._parse_updateCandidateParams(loadDict, candidateParams)
+
     if applyChanges:
       self.applyChanges()
-    self.lastAppliedName = Path(stateName).stem
-    self.stateSelector.setText(self.lastAppliedName)
+      if stateName:
+        stateName = Path(stateName).stem
+      self.lastAppliedName = stateName
+      self.stateSelector.setValue(self.lastAppliedName)
     return fns.paramValues(self.params, includeDefaults=True)
 
 
@@ -353,6 +364,7 @@ class ParamEditor(QtWidgets.QDockWidget):
                    overrideBasePath: t.Sequence[str]=None,
                    btnOpts: t.Union[PrjParam, dict]=None,
                    nest=True,
+                   parentParam: Parameter=None,
                    returnParam=False,
                    **kwargs):
     """
@@ -383,6 +395,7 @@ class ParamEditor(QtWidgets.QDockWidget):
       directly to the parent parameter specified by `namePath` + `baseRegisterPath`
     :param returnParam: Whether to return the parent parameter associated with this newly
       registered function
+    :param parentParam: Which parent should hold this registered function. If None, defaults to `self.param`
     :param kwargs: All additional kwargs are passed to AtomicProcess when wrapping the function.
     """
     if not isinstance(func, ProcessStage):
@@ -403,7 +416,9 @@ class ParamEditor(QtWidgets.QDockWidget):
     else:
       namePath = tuple(overrideBasePath) + tuple(namePath)
 
-    topParam = fns.getParamChild(self.params, *namePath)
+    if parentParam is None:
+      parentParam = self.params
+    topParam = fns.getParamChild(parentParam, *namePath)
     if len(proc.input.hyperParamKeys) > 0:
       # Check if proc params already exist from a previous addition
       wrapped = NestedProcWrapper(proc, topParam, paramFormat or fns.nameFormatter, treatAsAtomic=True, nestHyperparams=nest)
@@ -439,9 +454,8 @@ class ParamEditor(QtWidgets.QDockWidget):
     if not name.endswith('Tools'):
       name = name + ' Tools'
     toolsEditor = cls(fileType=lowerGroupName.replace(' ', ''), name=name)
-    for btn in (toolsEditor.saveAsBtn, toolsEditor.applyBtn, toolsEditor.expandAllBtn,
-                toolsEditor.collapseAllBtn):
-      btn.hide()
+    for widget in (toolsEditor.saveAsBtn, toolsEditor.applyBtn):
+      widget.hide()
     return toolsEditor
 
   def createMenuOpt(self, overrideName=None, parentMenu: QtWidgets.QMenu=None):
@@ -483,13 +497,12 @@ class ParamEditor(QtWidgets.QDockWidget):
   def updateSavedStates(self):
     dirGlob = list(self.saveDir.glob(f'*.{self.fileType}'))
     dirStates = [f.stem for f in dirGlob]
-    mod: QtCore.QStringListModel = self.stateSelector.model
 
     newStates = set(dirStates) - set(self.savedStates)
     delStates = set(self.savedStates) - set(newStates)
 
     self.savedStates = dirStates
-    mod.setStringList(self.savedStates)
+    self.stateSelector.setLimits(self.savedStates)
     for state in newStates:
       self.sigParamStateCreated.emit(state)
     for state in delStates:
@@ -544,29 +557,30 @@ class ParamEditor(QtWidgets.QDockWidget):
     self.saveAsBtn = QtWidgets.QPushButton('Save As...')
     self.applyBtn = QtWidgets.QPushButton('Apply')
 
-    self.stateSelector = PopupLineEditor(clearOnComplete=False, parent=self)
     """Shows current state or load a new one"""
-    def onReturn():
-      val = self.stateSelector.text()
+    def onReturn(param, val):
       if val in self.savedStates and val != self.lastAppliedName:
         self.loadParamValues(val)
-    self.stateSelector.returnPressed.connect(onReturn)
 
     # -----------
     # Widget layout
     # -----------
+    self.treeBtnsWidget = EasyWidget([self.expandAllBtn, self.collapseAllBtn])
+    self.loadStateWidgets, self.stateSelector = EasyWidget.fromPgParam(name='Load State', type='popuplineeditor')
+    self.stateSelector.sigValueChanged.connect(onReturn)
     children = [
-      [QtWidgets.QLabel('Load state: '), self.stateSelector],
-      [self.expandAllBtn, self.collapseAllBtn],
+      self.loadStateWidgets,
+      self.treeBtnsWidget,
       self.tree,
       [self.saveAsBtn, self.applyBtn]
     ]
 
-    self.dockContentsWidget = utilitys.widgets.EasyWidget.buildWidget(children)
+    self.dockContentsWidget = EasyWidget.buildWidget(children)
 
     if self.saveDir is None:
       self.saveAsBtn.hide()
-      self.dockContentsWidget.easyChild.children_[0].widget_.hide()
+      self.treeBtnsWidget.widget_.hide()
+      self.loadStateWidgets.widget_.hide()
 
     self.centralLayout = self.dockContentsWidget.easyChild.layout_
     self.setWidget(self.dockContentsWidget)
