@@ -1,10 +1,8 @@
 from __future__ import annotations
 
-import typing
 import warnings
-from functools import partial
 from pathlib import Path
-from typing import List, Optional
+from typing import Optional
 
 import numpy as np
 from pyqtgraph.Qt import QtGui, QtWidgets, QtCore
@@ -13,9 +11,9 @@ from pyqtgraph.parametertree.Parameter import PARAM_TYPES
 from pyqtgraph.parametertree.parameterTypes import ActionParameterItem, ActionParameter, \
   TextParameterItem, TextParameter
 
-from ..fns import clsNameOrGroup, params_flattened, popupFilePicker
 from .prjparam import PrjParam
 from .shortcut import ShortcutsEditor
+from ..fns import clsNameOrGroup, params_flattened, popupFilePicker
 from ..widgets import PopupLineEditor, ButtonCollection
 
 
@@ -164,32 +162,6 @@ class ShortcutKeySeqParameter(ActionParameter):
     self.isActivateConnected = False
 
 
-class CustomMenuParameter(parameterTypes.GroupParameter):
-  def __init__(self, menuActions: List[str]=None, **opts):
-    if menuActions is None:
-      menuActions: List[str] = []
-    self.menuActions = menuActions
-    self.item = None
-    super().__init__(**opts)
-
-  def makeTreeItem(self, depth):
-    item = super().makeTreeItem(depth)
-    self.item = item
-    if not hasattr(item, 'contextMenu'):
-      item.contextMenu = QtWidgets.QMenu()
-    item.contextMenuEvent = lambda ev: item.contextMenu.popup(ev.globalPos())
-    for actName in self.menuActions:
-      act = item.contextMenu.addAction(actName)
-      act.triggered.connect(partial(self.menuActTriggered, actName))
-    return item
-
-  def menuActTriggered(self, actName: str):
-    # Toggle 'enable' on click
-    return
-
-  def setOpts(self, **opts):
-    super().setOpts()
-
 class PopupLineEditorParameterItem(parameterTypes.WidgetParameterItem):
   def __init__(self, param, depth):
     strings = param.opts.get('limits', [])
@@ -213,49 +185,71 @@ class PopupLineEditorParameterItem(parameterTypes.WidgetParameterItem):
 class PopupLineEditorParameter(Parameter):
   itemClass = PopupLineEditorParameterItem
 
-_toggleName = 'Toggle Enable'
-class ProcGroupParameter(CustomMenuParameter):
-  def __init__(self, **opts):
-    menuActions = opts.pop('menuActions', [])
-    if _toggleName not in menuActions:
-      menuActions.append(_toggleName)
-    super().__init__(menuActions=menuActions, **opts)
+class ProcGroupParameterItem(parameterTypes.GroupParameterItem):
+
+  def __init__(self, param, depth):
+    self.enabledFontMap = None
+    super().__init__(param, depth)
+
+  def _mkFontMap(self):
+    if self.enabledFontMap:
+      return
+    allowDisable = self.param.opts['process'].allowDisable
+    if allowDisable:
+      enabledFont = self.font(0)
+    else:
+      enabledFont = QtGui.QFont()
+      enabledFont.setBold(False)
+
     disableFont = QtGui.QFont()
     disableFont.setStrikeOut(True)
-    self.enabledFontMap = {True: None, False: disableFont}
-    self.item = None
+    self.enabledFontMap = {True: enabledFont, False: disableFont}
 
-  def makeTreeItem(self, depth):
-    item = super().makeTreeItem(depth)
-    self.enabledFontMap[True] = QtGui.QFont(item.font(0))
-    item.setFont(0, self.enabledFontMap[self.opts['enabled']])
-    self.item = item
-    return item
 
-  def menuActTriggered(self, act: str):
-    item = self.item
-    # Toggle 'enable' on click
-    disabled = self.opts['enabled']
-    enabled = not disabled
-    item.setFont(0, self.enabledFontMap[enabled])
-    for ii in range(item.childCount()):
-      item.child(ii).setDisabled(disabled)
-    self.opts['enabled'] = enabled
+  def optsChanged(self, param, opts):
+    proc = param.opts['process']
+    if 'enabled' in opts:
+      enabled = opts['enabled']
+      if proc.allowDisable:
+        # Bypass subclass to prevent early short-circuit
+        if enabled:
+          super().setData(0, QtCore.Qt.CheckStateRole, QtCore.Qt.Checked)
+        else:
+          super().setData(0, QtCore.Qt.CheckStateRole, QtCore.Qt.Unchecked)
+      # This gets called before constructor can finish, so add enabled font map here
+      proc.disabled = not enabled
+      self._mkFontMap()
+      self.setFont(0, self.enabledFontMap[enabled])
+
+  def updateFlags(self):
+    # It's a shame super() doesn't return flags...
+    super().updateFlags()
+    opts = self.param.opts
+    flags = self.flags()
+    if opts['process'].allowDisable:
+      flags |= QtCore.Qt.ItemIsUserCheckable & (~QtCore.Qt.ItemIsTristate)
+    self.setFlags(flags)
+
+  def setData(self, column, role, value):
+    isChk = role == QtCore.Qt.CheckStateRole
+    # Check for invalid request to change check state
+    if isChk and self.param.parent() and not self.param.parent().opts['enabled']:
+      return
+    checkstate = self.checkState(column)
+    super().setData(column, role, value)
+    if isChk and int(checkstate) != value:
+      self.param.setOpts(enabled=bool(value))
+
+class ProcGroupParameter(parameterTypes.GroupParameter):
+  itemClass = ProcGroupParameterItem
 
   def setOpts(self, **opts):
-    enabled = opts.get('enabled', None)
-    if enabled is not None and enabled != self.opts['enabled']:
-      self.menuActTriggered(_toggleName)
+    enabled = opts.get('enabled')
+    curEnabled = self.opts['enabled']
     super().setOpts(**opts)
-
-
-class AtomicGroupParameter(parameterTypes.GroupParameter):
-  def makeTreeItem(self, depth):
-    item = super().makeTreeItem(depth)
-    font = QtGui.QFont()
-    font.setBold(False)
-    item.setFont(0, font)
-    return item
+    if enabled is not None and enabled != curEnabled:
+      for chParam in filter(lambda el: isinstance(el, ProcGroupParameter), self):
+        chParam.setOpts(enabled=enabled)
 
 class _DummySignal:
   def connect(self, *args): pass
@@ -287,10 +281,7 @@ class FilePickerParameterItem(parameterTypes.WidgetParameterItem):
 
   def _retrieveFolderName_gui(self):
     curVal = self.param.value()
-    if curVal:
-      useDir = curVal
-    else:
-      useDir = str(Path.cwd())
+    useDir = curVal or str(Path.cwd())
     opts = self.param.opts
     opts['startDir'] = str(Path(useDir).absolute())
     fname = popupFilePicker(None, 'Select File', **opts)
@@ -396,6 +387,7 @@ class ChecklistParameter(parameterTypes.GroupParameter):
     self.setOpts(**opts)
     if value is not None:
       self.setValue(value)
+    self.sigOptionsChanged.connect(lambda obj, opts: self.setOpts(**opts))
 
   def setLimits(self, limits):
     super().setLimits(limits)
@@ -404,10 +396,7 @@ class ChecklistParameter(parameterTypes.GroupParameter):
     self.clearChildren()
     for chOpts in limits:
       if isinstance(chOpts, str):
-        if exclusive:
-          newVal = oldOpts.get(chOpts, False)
-        else:
-          newVal = oldOpts.get(chOpts, True)
+        newVal = oldOpts.get(chOpts, not exclusive)
         chOpts = dict(name=chOpts, value=newVal)
       child = self.create(type='bool', **chOpts)
       self._hookupParam(child, exclusive)
@@ -463,7 +452,6 @@ class NoneParameter(parameterTypes.SimpleParameter):
 parameterTypes.registerParameterType('NoneType', NoneParameter)
 parameterTypes.registerParameterType('keyseq', KeySequenceParameter)
 parameterTypes.registerParameterType('procgroup', ProcGroupParameter)
-parameterTypes.registerParameterType('atomicgroup', AtomicGroupParameter)
 parameterTypes.registerParameterType('shortcutkeyseq', ShortcutKeySeqParameter)
 parameterTypes.registerParameterType('shortcut', ShortcutParameter)
 parameterTypes.registerParameterType('popuplineeditor', PopupLineEditorParameter)
