@@ -111,7 +111,7 @@ class ParamEditorPlugin(pe.EditorPropsMixin):
       ShortcutParameter.REGISTRY.registerAction(opts, act,
                                                 namePath=(owner.__groupingName__,))
 
-    act.triggered.connect(lambda: proc(win=self.win))
+    act.triggered.connect(proc)
     return proc
 
   def registerPopoutFuncs(self, funcList: Sequence[Callable], nameList: Sequence[str]=None, groupName:str=None, btnOpts: PrjParam=None):
@@ -138,6 +138,8 @@ class ParamEditorPlugin(pe.EditorPropsMixin):
   def attachWinRef(self, win):
     self.win = win
     self.menu.setParent(self.parentMenu, self.menu.windowFlags())
+    for proc in self.toolsEditor.procToParamsMapping:
+      proc.updateInput(win=win)
 
 
 def dockPluginFactory(name_: str=None, editors: Sequence[pe.ParamEditor]=None):
