@@ -2,7 +2,7 @@ from __future__ import annotations
 
 from functools import singledispatch
 from functools import wraps
-from typing import List, Sequence, Callable
+from typing import List, Sequence, Callable, Union
 
 from pyqtgraph.parametertree import Parameter
 from pyqtgraph.parametertree.parameterTypes import GroupParameter
@@ -14,52 +14,6 @@ from ..processing import *
 
 __all__ = ['NestedProcWrapper']
 
-def atomicRunWrapper(proc: AtomicProcess, names: Sequence[str], params: Sequence[Parameter]):
-  oldRun = proc.run
-  @wraps(oldRun)
-  def newRun(io: ProcessIO = None, disable=False, **runKwargs) -> ProcessIO:
-    newIo = {name: param.value() for name, param in zip(names, params)}
-    proc.input.update(**newIo)
-    return oldRun(io, disable, **runKwargs)
-  return newRun
-
-def procRunWrapper(proc: NestedProcess, groupParam: Parameter):
-  oldRun = proc.run
-  @wraps(oldRun)
-  def newRun(io: ProcessIO = None, disable=False, **runKwargs):
-    proc.disabled = not groupParam.opts['enabled']
-    return oldRun(io, disable=disable, **runKwargs)
-  return newRun
-
-@singledispatch
-def addStageToParam(stage: ProcessStage, parentParam: Parameter, **kwargs):
-  pass
-
-@addStageToParam.register
-def addAtomicToParam(stage: AtomicProcess, parentParam: Parameter,
-                 argNameFormat: Callable[[str], str]=None, **kwargs):
-  docParams = fns.docParser(stage.func.__doc__)
-  parentParam.setOpts(tip=docParams['top-descr'])
-  params: List[Parameter] = []
-  for key in stage.input.hyperParamKeys:
-    val = stage.input[key]
-    curParam = docParams.get(key, None)
-    if curParam is None:
-      curParam = PrjParam(name=key, value=val)
-    else:
-      if val is not stage.input.FROM_PREV_IO:
-        curParam.value = val
-      if curParam.pType is None:
-        curParam.pType = type(val).__name__
-    paramDict = curParam.toPgDict()
-    if argNameFormat is not None and 'title' not in paramDict:
-      paramDict['title'] = argNameFormat(key)
-    pgParam = fns.getParamChild(parentParam, chOpts=paramDict)
-    _hookupCondition(parentParam, pgParam)
-    params.append(pgParam)
-  stage.run = atomicRunWrapper(stage, stage.input.hyperParamKeys, params)
-  return stage
-
 def _hookupCondition(parentParam: Parameter, chParam: Parameter):
   condition = chParam.opts.get('condition', None)
   if not condition:
@@ -85,60 +39,117 @@ def _hookupCondition(parentParam: Parameter, chParam: Parameter):
   if ch:
     onChanged(ch, ch.value())
 
+def _mkProcParam(stage: ProcessStage):
+  return {'name': stage.name, 'type': 'procgroup', 'process': stage, 'enabled': not stage.disabled}
+
+def atomicRunWrapper(proc: AtomicProcess, param: Parameter):
+  param.sigValueChanged.connect(lambda p, val: proc.updateInput(graceful=True, **{param.name(): val}))
+
+@singledispatch
+def addStageToParam(stage: ProcessStage, parentParam: Parameter, **kwargs):
+  pass
+
+@addStageToParam.register
+def addAtomicToParam(stage: AtomicProcess, parentParam: Parameter,
+                     argNameFormat: Callable[[str], str]=None, nestHyperparams=True, **kwargs):
+  docParams = fns.docParser(stage.func.__doc__)
+  if nestHyperparams:
+    parentParam = fns.getParamChild(parentParam, chOpts=_mkProcParam(stage))
+  parentParam.setOpts(tip=docParams['top-descr'])
+  for key in stage.input.hyperParamKeys:
+    val = stage.input[key]
+    curParam = docParams.get(key, None)
+    if curParam is None:
+      curParam = PrjParam(name=key, value=val)
+    else:
+      if val is not stage.input.FROM_PREV_IO:
+        curParam.value = val
+      if curParam.pType is None:
+        curParam.pType = type(val).__name__
+    paramDict = curParam.toPgDict()
+    if argNameFormat is not None and 'title' not in paramDict:
+      paramDict['title'] = argNameFormat(key)
+    pgParam = fns.getParamChild(parentParam, chOpts=paramDict)
+    atomicRunWrapper(stage, pgParam)
+    _hookupCondition(parentParam, pgParam)
+  return stage
 
 @addStageToParam.register
 def addNestedToParam(stage: NestedProcess, parentParam: Parameter, nestHyperparams=True,
                      argNameFormat: Callable[[str], str]=None, treatAsAtomic=False, **kwargs):
+  if nestHyperparams:
+    parentParam = fns.getParamChild(parentParam, chOpts=_mkProcParam(stage))
   if treatAsAtomic:
     collapsed = AtomicProcess(stage.run, stage.name, mainResultKeys=stage.mainResultKeys,
                               mainInputKeys=stage.mainInputKeys)
     collapsed.input = stage.input
     addAtomicToParam(collapsed, parentParam, argNameFormat)
     return
-  stage.run = procRunWrapper(stage, parentParam)
   # Special case of a process comprised of just one atomic function
-  if len(stage.stages) == 1 and isinstance(stage.stages[0], AtomicProcess):
-    # isinstance ensures the type will be correct
-    # noinspection PyTypeChecker
-    addAtomicToParam(stage.stages[0], parentParam)
+  if len(stage.stages) == 1:
+    # Hide this level of the process tree
+    addStageToParam(stage.stages[0], parentParam, nestHyperparams=False)
     return
-  outerParent = parentParam
-  for childStage in stage.stages:
-    pType = 'atomicgroup'
-    if childStage.allowDisable:
-      pType = 'procgroup'
-    if nestHyperparams:
-      paramDict = PrjParam(name=childStage.name, pType=pType, value=[],
-                          enabled=not childStage.disabled).toPgDict()
-      parentParam = fns.getParamChild(outerParent, chOpts=paramDict)
-    else:
-      parentParam = outerParent
+  for childStage in stage:
     addStageToParam(childStage, parentParam)
 
 class NestedProcWrapper(CompositionMixin):
-  def __init__(self, processor: ProcessStage, parentParam: GroupParameter=None,
+  def __init__(self, processor: NestedProcess, parentParam: GroupParameter=None,
                argNameFormat: Callable[[str], str] = None, treatAsAtomic=False, nestHyperparams=True):
-    self.processor = self.exposes(processor)
+    self.processor = self.exposes(processor, 'processor')
     self.algName = processor.name
     self.argNameFormat = argNameFormat
     self.treatAsAtomic = treatAsAtomic
     self.nestHyperparams = nestHyperparams
+    # A few special things happen when adding a top-level processor. It has to create a parent if none exists without
+    # doubly nesting and return a reference to the created parent param or the normal one if no nesting occurred.
+    # The if-branching allows this to occur
     if parentParam is None:
-      parentParam = Parameter.create(name=self.algName, type='group')
-    elif nestHyperparams:
-      parentParam = fns.getParamChild(parentParam, self.algName)
+      parentParam = Parameter.create(name=self.algName, type='procgroup')
+      # Avoid doubly-nested top parameter
+      self.nestHyperparams = False
     self.parentParam : GroupParameter = parentParam
-    self.addStage(self.processor)
 
-  def addStage(self, stage: ProcessStage):
-    addStageToParam(stage, self.parentParam, argNameFormat=self.argNameFormat,
+    self.addStage(self.processor)
+    # Extract nested param if it was created
+    if  self.nestHyperparams:
+      self.parentParam = parentParam.child(self.algName)
+
+    # Reset once more if no initial parent param
+    self.nestHyperparams = nestHyperparams
+
+
+  def addStage(self, stage: ProcessStage, before: Sequence[str]=()):
+    parentParam = fns.getParamChild(self.parentParam, *before, allowCreate=False)
+    parentStage: NestedProcess = self.getNestedName(self.processor, *before[:-1])
+    # Defensive check to only add extra stages onto existing processor
+    if stage is not self.processor:
+      if before:
+        tmpStage = self.getNestedName(parentStage, before[-1])
+        beforeIdx = parentStage.stages.index(tmpStage)
+        parentStage.stages = parentStage.stages[:beforeIdx] + [stage] + parentStage.stages[beforeIdx:]
+      else:
+        parentStage.stages.append(stage)
+    addStageToParam(stage, parentParam, argNameFormat=self.argNameFormat,
                     treatAsAtomic=self.treatAsAtomic, nestHyperparams=self.nestHyperparams)
 
+  def removeStage(self, *nestedName: str):
+    parent = self.getNestedName(self.processor, *nestedName[:-1])
+    stage = self.getNestedName(parent, nestedName[-1])
+    parent.stages.remove(stage)
+    parentParam = fns.getParamChild(self.parentParam, *nestedName[:-1], allowCreate=False)
+    childParam = fns.getParamChild(parentParam, nestedName[-1], allowCreate=False)
+    parentParam.removeChild(childParam)
+
+  def clear(self):
+    for stage in self.processor.stages:
+      self.removeStage(stage.name)
+
   def setStageEnabled(self, stageIdx: Sequence[str], enabled: bool):
     paramForStage = self.parentParam.child(*stageIdx)
     prevEnabled = paramForStage.opts['enabled']
     if prevEnabled != enabled:
-      paramForStage.menuActTriggered('Toggle Enable')
+      paramForStage.setOpts(enabled=enabled)
 
   def __repr__(self) -> str:
     selfCls = type(self)
@@ -149,13 +160,15 @@ class NestedProcWrapper(CompositionMixin):
     return oldName
 
   @classmethod
-  def getNestedName(cls, curProc: ProcessStage, nestedName: List[str]):
-    if len(nestedName) == 0 or isinstance(curProc, AtomicProcess):
+  def getNestedName(cls, curProc: ProcessStage, *nestedName: str):
+    if not nestedName or isinstance(curProc, AtomicProcess):
       return curProc
     # noinspection PyUnresolvedReferences
-    for stage in curProc.stages:
+    for stage in curProc:
       if stage.name == nestedName[0]:
         if len(nestedName) == 1:
           return stage
         else:
-          return cls.getNestedName(stage, nestedName[1:])
\ No newline at end of file
+          return cls.getNestedName(stage, *nestedName[1:])
+    # Requested stage not found if execution reaches here
+    raise ValueError(f'Stage {nestedName} not found in {curProc}')
\ No newline at end of file
