@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-import copy
+import copy as _copy
 import inspect
 import typing as t
 from abc import ABC, abstractmethod
@@ -51,7 +51,7 @@ class ProcessIO(dict):
       if k not in kwargs:
         warnKeys.append(k)
         kwargs[k] = None
-    if len(warnKeys) > 0:
+    if warnKeys:
       warn(f'Hyperparameter keys were specified, but did not exist in provided'
            f' inputs:\n{warnKeys}\n'
            f'Defaulting to `None` for those keys.', UserWarning)
@@ -121,10 +121,19 @@ class ProcessStage(ABC):
   def __str__(self) -> str:
     return repr(self)
 
-  def updateInput(self, prevIo: ProcessIO=None, **kwargs):
+  def __eq__(self, other: NestedProcess):
+    return self.saveState(True, True) == other.saveState(True, True)
+
+  def __hash__(self):
+    return id(self)
+
+  def updateInput(self, prevIo: ProcessIO=None, graceful=False, **kwargs):
     """
     Helper function to update current inputs from previous ones while ignoring leading
     underscores.
+
+    :param prevIo: Io object to forward updated inputs here. Extra inputs can be supplied to **kwargs
+    :param graceful: If *True*, doesn't error on missing keys or bad update hierarchy
     """
     if prevIo is None:
       prevIo = ProcessIO()
@@ -139,7 +148,7 @@ class ProcessStage(ABC):
         self.input[trueK] = useIo[prevIoKeyToFmt[fmtK]]
       elif fmtK in requiredKeyFmt:
         missingKeys.append(fmtK)
-    if len(missingKeys) > 0:
+    if missingKeys and not graceful:
       raise KeyError(f'Missing Following keys from {self}: {missingKeys}')
 
   def run(self, io: ProcessIO=None, disable=False, **runKwargs):
@@ -148,11 +157,55 @@ class ProcessStage(ABC):
   def __call__(self, **kwargs):
     return self.run(ProcessIO(**kwargs))
 
-  @property
-  @abstractmethod
-  def stages_flattened(self):
+  def __iter__(self):
     raise NotImplementedError
 
+  def saveState(self, includeDefaults=False, includeMeta=False):
+    """
+    Serializes the process' state in terms of its composition and inputs.
+
+    :param includeDefaults: Whether to also serialize inputs that are unchanged from the original process creation
+    :param includeMeta: Whether to also serialize metadata about the stage such as its disable/allowDisable status
+    """
+    raise NotImplementedError
+
+  def saveState_flattened(self, includeDefault=False, includeMeta=False):
+    """Saves state while collapsing all nested processes into one list of atomic processes"""
+    return self.saveState(includeDefault, includeMeta)
+
+  def saveMetaProps(self, **filterOpts):
+    """
+    Converts a saved state of parameters and staegs to one including disabled and allowDisable statuses
+
+    :param filterOpts: Values to ignore if they match. So, passing `disabled=True` will only
+      record the `disabled` property if it is *False*. This is an easy way of ignoring default properties. Setting a key
+      to *None* will ignore that property entirely.
+    """
+    metaProps = ('allowDisable', 'disabled')
+    # Convert back if no props were added
+    out = {}
+    for k in metaProps:
+      selfVal = getattr(self, k)
+      cmp = filterOpts.get(k)
+      if cmp is None or np.array_equal(cmp, selfVal):
+        continue
+      out[k] = selfVal
+    return out
+
+  def addMetaProps(self, state: t.Union[dict, str], **metaFilterOpts):
+    """
+    Helper method to insert meta properties into the current state. Since the state might be a string, or meta props
+    might be empty, this performs checks to ensure only the most simplified output form is preserved
+    """
+    state = _copy.copy(state)
+    metaProps = self.saveMetaProps(**metaFilterOpts)
+    if not metaProps:
+      return state
+    if isinstance(state, str):
+      state = {state: {}}
+    state.update(metaProps)
+    return state
+
 class AtomicProcess(ProcessStage):
   """
   Often, process functions return a single argument (e.g. string of text,
@@ -200,6 +253,7 @@ class AtomicProcess(ProcessStage):
     if needsWrap:
       func = self._wrappedFunc(func, self.mainResultKeys)
     self.func = func
+    self.defaultInput = self.input.copy()
 
   @classmethod
   def _wrappedFunc(cls, func, mainResultKeys: StrList=None):
@@ -225,17 +279,29 @@ class AtomicProcess(ProcessStage):
     return self.input.keysFromPrevIO
 
   def run(self, prevIO: ProcessIO=None, disable=False, **runKwargs):
+    ProcessIO.count = self
+    disable = disable or self.disabled
     if prevIO is not None:
       self.updateInput(prevIO, **runKwargs)
-    if not disable:
-      self.result = self.func(**self.input)
-    else:
-      self.result = self.input
+    self.result = self.func(**self.input) if not disable else self.input
     return self.result
 
-  @property
-  def stages_flattened(self):
-    return [self]
+  def saveState(self, includeDefaults=False, includeMeta=False, **metaFilterOpts):
+    def keepCond(k, v):
+      return (k in self.input.hyperParamKeys or k not in self.defaultInput) \
+              and (includeDefaults or v != self.defaultInput.get(k)) \
+              and v is not self.input.FROM_PREV_IO
+    saveIo = {k: v for k, v in self.input.items() if keepCond(k, v)}
+    if not saveIo:
+      state = self.name
+    else:
+      state = {self.name: saveIo}
+    if includeMeta:
+      state = self.addMetaProps(state, **metaFilterOpts)
+    return state
+
+  def __iter__(self):
+    return iter([])
 
 class NestedProcess(ProcessStage):
 
@@ -243,6 +309,7 @@ class NestedProcess(ProcessStage):
     self.stages: t.List[ProcessStage] = []
     self.name = name
     self.allowDisable = True
+    self.result = ProcessIO()
     if mainInputKeys is not None:
       self.mainInputKeys = mainInputKeys
     if mainResultKeys is not None:
@@ -260,10 +327,8 @@ class NestedProcess(ProcessStage):
     if keySpec is None:
       keySpec = self
     atomic = AtomicProcess(func, mainResultKeys=keySpec.mainResultKeys, mainInputKeys=keySpec.mainInputKeys, **kwargs)
-    numSameNames = 0
-    for stage in self.stages:
-      if atomic.name == stage.name.split('#')[0]:
-        numSameNames += 1
+    numSameNames = sum(
+        atomic.name == stage.name.split('#')[0] for stage in self)
     if numSameNames > 0:
       atomic.name = f'{atomic.name}#{numSameNames+1}'
     if self.name is None:
@@ -284,41 +349,93 @@ class NestedProcess(ProcessStage):
     self.stages.append(process)
     return process
 
+  def updateInput(self, prevIo: ProcessIO=None, graceful=False, **kwargs):
+    """Drill down to the necessary stages when setting input values when setter values are dicts"""
+    if prevIo is None:
+      prevIo = ProcessIO()
+    prevIo.update(kwargs)
+    for kk in list(prevIo):
+      # List wrap instead of items() to avoid mutate error
+      vv = prevIo[kk]
+      if isinstance(vv, dict):
+        matchStage = [s for s in self if s.name == kk]
+        if not matchStage and not graceful:
+          raise ValueError(f'Name {kk} doesn\'t match existing stages')
+        elif not matchStage:
+          continue
+        matchStage[0].updateInput(**vv, graceful=graceful)
+        prevIo.pop(kk)
+    # Normal input maps to first stage, so propagate all normal args there
+    self.stages[0].updateInput(prevIo, graceful=graceful)
+
   def run(self, io: ProcessIO = None, disable=False, **runKwargs):
-    if io is None:
-      _activeIO = ProcessIO()
-    else:
-      _activeIO = copy.copy(io)
+    _activeIO = ProcessIO() if not io else _copy.copy(io)
     _activeIO.update(runKwargs)
 
-    for i, stage in enumerate(self.stages):
-      newIO = stage.run(_activeIO, disable=self.disabled or disable)
+    if self.disabled or disable and not self.result:
+      # TODO: Formalize this behavior. Disabling before being run should avoid eating the input args
+      return _activeIO
+
+    for i, stage in enumerate(self):
+      newIO = stage.run(_activeIO, disable=disable or self.disabled)
       if isinstance(newIO, ProcessIO):
         _activeIO.update(newIO)
 
+    self.result = _activeIO
     return self.result
 
-  @property
-  def result(self):
-    return self.stages[-1].result
+  def saveState(self, includeDefaults=False, includeMeta=False, **metaFilterOpts):
+    stageStates = []
+    for stage in self:
+      curState = stage.saveState(includeDefaults, includeMeta)
+      stageStates.append(curState)
+    state = {self.name: stageStates}
+    if includeMeta:
+      state = self.addMetaProps(state, **metaFilterOpts)
+    return state
+
+  def saveState_flattened(self, **kwargs):
+    """Saves state while collapsing all nested processes into one list of atomic processes"""
+    return self.flatten().saveState(**kwargs)
 
   @property
   def input(self):
     return self.stages[0].input
 
+  def flatten(self, copy=True, includeDisabled=True):
+    if copy:
+      outProc = type(self)(self.name, self.mainInputKeys, self.mainResultKeys)
+    else:
+      outProc = self
+    outProc.stages = self._getFlatStages(includeDisabled)
+    return outProc
+
   @property
   def stages_flattened(self):
+    """Property version of flattened self which, like `stages`, will always include disabled stages"""
+    return self._getFlatStages()
+
+  def _getFlatStages(self, includeDisabled=True):
     outStages: t.List[ProcessStage] = []
-    for stage in self.stages:
-      outStages.extend(stage.stages_flattened)
+    for stage in self:
+      if stage.disabled and not includeDisabled:
+        continue
+      if isinstance(stage, AtomicProcess):
+        outStages.append(stage)
+      else:
+        stage: NestedProcess
+        outStages.extend(stage._getFlatStages(includeDisabled))
     return outStages
 
+  def __iter__(self):
+    return iter(self.stages)
+
   def _stageSummaryWidget(self):
     raise NotImplementedError
 
   def _nonDisabledStages_flattened(self):
     out = []
-    for stage in self.stages:
+    for stage in self:
       if isinstance(stage, AtomicProcess):
         out.append(stage)
       elif not stage.disabled:
@@ -378,7 +495,7 @@ class NestedProcess(ProcessStage):
     """
     validInfos = []
     for info in infos:
-      validInfo = copy.copy(info)
+      validInfo = _copy.copy(info)
       for lastInfo in prevInfos:
         for key in set(info.keys()).intersection(lastInfo.keys()) - {'name'}:
           if np.array_equal(info[key], lastInfo[key]):
