@@ -20,7 +20,7 @@ from typing_extensions import Literal
 from numpy.distutils.misc_util import is_sequence
 
 from utilitys.fns import nameFormatter, pascalCaseToTitle, params_flattened, \
-  dynamicDocstring, getParamChild
+  dynamicDocstring, getParamChild, hookupParamWidget
 from utilitys.misc import CompositionMixin
 from utilitys.typeoverloads import FilePath
 from . import params
@@ -385,8 +385,14 @@ class EasyWidget:
                layout: str=None, useSplitter=False, baseWidget: QtWidgets.QWidget=None):
     if baseWidget is None:
       baseWidget = QtWidgets.QWidget()
+    self._built = False
     self.children_ = children
-    self.useSplitter = useSplitter
+    self.useSplitter = None
+    self.widget_ = baseWidget
+
+    self._resetOpts(useSplitter, layout)
+
+  def _resetOpts(self, useSplitter, layout):
     if layout == 'V':
       orient = QtCore.Qt.Vertical
       layout = QtWidgets.QVBoxLayout
@@ -397,19 +403,27 @@ class EasyWidget:
       orient = layout = None
     self.orient_ = orient
 
+    if useSplitter == self.useSplitter and self.layout_:
+      return
+    # Had children in existing widget which will be discarded when changing self widget_ to splitter
+    if self.widget_.children() and useSplitter:
+      raise ValueError("Cannot change splitter status to *True* when widget already has children")
+    self.useSplitter = useSplitter
+
     if useSplitter:
       self.layout_ = QtWidgets.QSplitter(orient)
       self.widget_ = self.layout_
     else:
-      self.widget_ = baseWidget
       try:
         self.layout_ = layout()
         self.widget_.setLayout(self.layout_)
       except TypeError:
-        # When win is none
+        # When layout is none
         self.layout_ = None
 
   def build(self):
+    if self._built:
+      return
     if self.layout_ is None:
       raise ValueError('Top-level orientation must be set to "V" or "H" before adding children')
     if self.orient_ == QtCore.Qt.Horizontal:
@@ -423,6 +437,7 @@ class EasyWidget:
       morphChild = self.addChild(child, chSuggested)
       if morphChild is not child:
         self.children_[ii] = morphChild
+    self._built = True
 
   def addChild(self, child: t.Union[QtWidgets.QWidget, t.Sequence, EasyWidget], suggestedLayout:str=None):
     if isinstance(child, QtWidgets.QWidget):
@@ -434,12 +449,21 @@ class EasyWidget:
       self.layout_.addWidget(child.widget_)
     return child
 
+  def insertChild(self, child: EasyWidget, index: int):
+    child.build()
+    return self.layout_.insertWidget(index, child.widget_)
+
+  def hide(self):
+    self.widget_.hide()
+  def show(self):
+    self.widget_.show()
+
   @classmethod
   def listChildrenWrapper(cls, children: t.Union[t.Sequence, EasyWidget], maybeNewLayout: str=None):
     if not isinstance(children, EasyWidget):
       children = cls(children)
     if children.layout_ is None and maybeNewLayout is not None:
-      children = cls(children.children_, maybeNewLayout, children.useSplitter)
+      children._resetOpts(children.useSplitter, maybeNewLayout)
     return children
 
   @classmethod
@@ -463,11 +487,39 @@ class EasyWidget:
     retWidget.easyChild = builder
     return retWidget
 
+  @classmethod
+  def fromPgParam(cls, param: Parameter=None, layout='H', **opts):
+    """
+    Creates a form-style EasyWidget (name + edit widget) from pyqtgraph parameter options or a parameter directly.
+
+    :param param: Parameter to use, if it already exists. Otherwise, one is created from `opts` and returned.
+    :param layout: EasyWidget layout
+    :param opts: If `param` is unspecified, a parameter is created from these opts instead and returned
+
+    :return: Just the EasyWidget if `param` is provided, otherwise (EasyWidget, Parameter) tuple
+    """
+    returnParam = False
+    if param is None:
+      param = Parameter.create(**opts)
+      returnParam = True
+    try:
+      item = param.itemClass(param, 0)
+      editWidget = item.makeWidget()
+      hookupParamWidget(param, editWidget)
+    except AttributeError as ex:
+      raise ValueError('Can only create EasyWidgets from parameters that have an itemClass and implement makeWidget().'
+                       f' Problem type: {opts["type"]}')
+    lbl = QtWidgets.QLabel(opts['name'])
+    obj = cls([lbl, editWidget], layout)
+    obj.build()
+    if returnParam:
+      return obj, param
+    return obj
+
 class HasEasyChild(QtWidgets.QMainWindow):
   """Provided just for type checking purposes"""
   easyChild: EasyWidget
 
-
 class ImageViewer(CompositionMixin, pg.PlotWidget):
   sigMouseMoved = QtCore.Signal(object) # ndarray(int, int) xy pos rel. to image
 
